package java.time.chrono;

import java.time.format.ResolverStyle;
import java.util.Map;
import java.time.temporal.TemporalQuery;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.TemporalField;
import java.time.format.DateTimeFormatterBuilder;
import java.time.format.TextStyle;
import java.time.temporal.ValueRange;
import java.time.temporal.ChronoField;
import java.util.List;
import java.time.Instant;
import java.time.DateTimeException;
import java.time.LocalTime;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.Clock;
import java.util.Set;
import java.util.Locale;
import java.time.temporal.TemporalQueries;
import java.util.Objects;
import java.time.temporal.TemporalAccessor;

public interface Chronology extends Comparable<Chronology>
{
    public static Chronology from(TemporalAccessor obj) {
        Objects.requireNonNull(obj, "temporal");
        Chronology chronology = (Chronology)obj.query(TemporalQueries.chronology());
        return (chronology != null) ? chronology : IsoChronology.INSTANCE;
    }
    
    public static Chronology ofLocale(Locale locale) {
        return AbstractChronology.ofLocale(locale);
    }
    
    public static Chronology of(String s) {
        return AbstractChronology.of(s);
    }
    
    public static Set<Chronology> getAvailableChronologies() {
        return AbstractChronology.getAvailableChronologies();
    }
    
    public abstract String getId();
    
    public abstract String getCalendarType();
    
    default public ChronoLocalDate date(Era era, int n, int n2, int n3) {
        return this.date(this.prolepticYear(era, n), n2, n3);
    }
    
    public abstract ChronoLocalDate date(int p0, int p1, int p2);
    
    default public ChronoLocalDate dateYearDay(Era era, int n, int n2) {
        return this.dateYearDay(this.prolepticYear(era, n), n2);
    }
    
    public abstract ChronoLocalDate dateYearDay(int p0, int p1);
    
    public abstract ChronoLocalDate dateEpochDay(long p0);
    
    default public ChronoLocalDate dateNow() {
        return this.dateNow(Clock.systemDefaultZone());
    }
    
    default public ChronoLocalDate dateNow(ZoneId zone) {
        return this.dateNow(Clock.system(zone));
    }
    
    default public ChronoLocalDate dateNow(Clock clock) {
        Objects.requireNonNull(clock, "clock");
        return this.date(LocalDate.now(clock));
    }
    
    public abstract ChronoLocalDate date(TemporalAccessor p0);
    
    default public ChronoLocalDateTime<? extends ChronoLocalDate> localDateTime(TemporalAccessor temporal) {
        try {
            return this.date(temporal).atTime(LocalTime.from(temporal));
        }
        catch (DateTimeException cause) {
            throw new DateTimeException("Unable to obtain ChronoLocalDateTime from TemporalAccessor: " + temporal.getClass(), cause);
        }
    }
    
    default public ChronoZonedDateTime<? extends ChronoLocalDate> zonedDateTime(TemporalAccessor temporalAccessor) {
        try {
            ZoneId from = ZoneId.from(temporalAccessor);
            try {
                return this.zonedDateTime(Instant.from(temporalAccessor), from);
            }
            catch (DateTimeException ex) {
                return ChronoZonedDateTimeImpl.ofBest(ChronoLocalDateTimeImpl.ensureValid(this, this.localDateTime(temporalAccessor)), from, null);
            }
        }
        catch (DateTimeException cause) {
            throw new DateTimeException("Unable to obtain ChronoZonedDateTime from TemporalAccessor: " + temporalAccessor.getClass(), cause);
        }
    }
    
    default public ChronoZonedDateTime<? extends ChronoLocalDate> zonedDateTime(Instant instant, ZoneId zoneId) {
        return ChronoZonedDateTimeImpl.ofInstant(this, instant, zoneId);
    }
    
    public abstract boolean isLeapYear(long p0);
    
    public abstract int prolepticYear(Era p0, int p1);
    
    public abstract Era eraOf(int p0);
    
    public abstract List<Era> eras();
    
    public abstract ValueRange range(ChronoField p0);
    
    default public String getDisplayName(TextStyle textStyle, Locale locale) {
        return new DateTimeFormatterBuilder().appendChronologyText(textStyle).toFormatter(locale).format(new TemporalAccessor() {
            @Override
            public boolean isSupported(TemporalField temporalField) {
                return false;
            }
            
            @Override
            public long getLong(TemporalField obj) {
                throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
            }
            
            @Override
            public <R> R query(TemporalQuery<R> query) {
                if (query == TemporalQueries.chronology()) {
                    return Chronology.this;
                }
                return super.query(query);
            }
        });
    }
    
    public abstract ChronoLocalDate resolveDate(Map<TemporalField, Long> p0, ResolverStyle p1);
    
    default public ChronoPeriod period(int n, int n2, int n3) {
        return new ChronoPeriodImpl(this, n, n2, n3);
    }
    
    public abstract int compareTo(Chronology p0);
    
    public abstract boolean equals(Object p0);
    
    public abstract int hashCode();
    
    public abstract String toString();
}
